// file: $DT/class/tree/v1.0/dt_bayes_0.cc
//

// isip include files
//
#include "attribute_node.h"
#include "tree_node.h"
#include "tree.h"
#include "decision_tree.h"

#include <integral_constants.h>
#include "attribute_node_constants.h"
#include "tree_node_constants.h"

// method: discrete_bayes_split_cc
//
// arguments:
//  int_4* selected_mask_a: (input) best mask
//  int_4 num_categories_a: (input) number of categories in an attribute
//  Attribute_node* attr_node_a: (input) the attribute node we are interested
//                               in
//  Tree_node* node_a: (input) the tree node we are currently trying to split
//
// return: logical_1 indicating success
//
// this method performs bayes splitting rule for discrete values
// and returns the value associated with the split
//
float_4 Decision_tree::discrete_bayes_split_cc(int_4* selected_mask_a,
					       int_4 num_categories_a,
					       Attribute_node* attr_node_a,
 					       Tree_node* node_a) {

  // get the class label
  //
  char_1* class_label = new char_1[ISIP_MAX_STRING_LENGTH];
  if (class_node_d->get_label_cc(class_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"discrete_bayes_split_cc",
                     (char_1*)"unable to get class label from class node");
  }

  // get the number of classes
  //
  int_4 num_classes;
  if (class_node_d->get_num_values_cc(num_classes) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
                     (char_1*)"unable to get the number of classes");
  }

  // get the attribute label
  //
  char_1* attr_label = new char_1 [ISIP_MAX_STRING_LENGTH];
  if (attr_node_a->get_label_cc(attr_label) != ISIP_TRUE) {
    error_handler_cc((char_1*)"discrete_bayes_split_cc",
                     (char_1*)"unable to get the label for this attribute");
  }

  // get the attribute values for this attribute
  //
  char_1** attr_values = new char_1* [num_categories_a];
  for (int_4 i=0; i<num_categories_a; i++) {
    attr_values[i] = new char_1 [ISIP_MAX_STRING_LENGTH];
  }
  if (attr_node_a->get_values_cc(attr_values) != ISIP_TRUE) {
    error_handler_cc((char_1*)"discrete_bayes_split_cc",
                     (char_1*)"unable to get the values of attributes");
  }

  // number of possible permutations of the categories
  //
  int_4 init;
  int_4 iter;
  calculate_loop_values_cc(init, iter, num_categories_a);

  int_4* mask = new int_4[num_categories_a];

  float_4 max_prob = DT_MINIMUM_FLOAT_4;
  
  // loop through all possible permutations given the type of search
  //
  for (int_4 i=init; i<iter; i++) {
    
    // create masks of 0, 1 for each possible permutation
    //
    if (mode_search_d == DT_SEARCH_COMPLETE) {
      permute_cc(i, mask, num_categories_a);
    }
    else if (mode_search_d == DT_SEARCH_FAST) {
      mask_single_cc(i, mask, num_categories_a);
    }

    // calculate the left tree prob if this split is taken
    //
    float_4 left_prob = calculate_bayes_left_prob_cc(mask,
					     class_label, num_classes,
					     attr_label, num_categories_a,
					     attr_values, node_a);
    // calculate the right tree prob if this split is taken
    //
    float_4 right_prob = calculate_bayes_right_prob_cc(mask,
					     class_label, num_classes,
					     attr_label, num_categories_a,
					     attr_values, node_a);

    float_4 total_prob = left_prob + right_prob;

    // keep the maximum split
    //
    if (total_prob > max_prob ) {
      max_prob = total_prob;
      for (int_4 k=0; k<num_categories_a; k++ ) {
	selected_mask_a[k] = mask[k];
      }
    }
  }

  // clean up
  //
  if (class_label != (char_1*)NULL) {
    delete [] class_label;
    class_label = (char_1*)NULL;
  }

  if (attr_label != (char_1*)NULL) {
    delete [] attr_label;
    attr_label = (char_1*)NULL;
  }

  if (attr_values != (char_1**)NULL) {
    for (int_4 i = 0; i < num_categories_a; i++) {
      if (attr_values[i] != (char_1*)NULL) {
	delete [] attr_values[i];
	attr_values[i] = (char_1*)NULL;
      }
    }
    delete [] attr_values;
    attr_values = (char_1**)NULL;
  }
  
  if (mask != (int_4*)NULL) {
    delete [] mask;
    mask = (int_4*)NULL;
  }

  // return maximum value 
  //
  return max_prob;
}



