// file: $DT/class/tree/v1.0/dt_bayes_4.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "tree_node_constants.h"
#include "tree_node.h"
#include "tree.h"
#include "attribute_node_constants.h"
#include "attribute_node.h"
#include "decision_tree.h"
#include <integral_constants.h>

// method: calculate_bayes_left_prob_cc
//
// arguments:
//  float_4 split_value_a: (input) value to determine the split
//  
//
// return: logical_1 indicating success
//
// this method 
//
float_4 Decision_tree::calculate_bayes_left_prob_cc (float_4 split_value_a,
						     char_1* class_label_a,
						     int_4 num_classes_a,
						     char_1* attr_label_a,
						     Tree_node* node_a) {

  // get the linked list containing the data nodes for this tree node
  //
  Linked_list* data_list;
  if (node_a->get_data_nodes_cc(data_list) != ISIP_TRUE) {
    error_handler_cc((char_1*)"continous_bayes_split_cc",
		     (char_1*)"unable to get data nodes from in tree node");
  }

  // loop over all the data nodes
  //
  char_1* class_value = new char_1 [ISIP_MAX_STRING_LENGTH];
  int_4* class_count = new int_4[num_classes_a];
  memset (class_count, 0, sizeof(int_4) * num_classes_a);
  float_4 attr_value;
  int_4 left_count = 0;

  while (data_list != (Linked_list*)NULL) {
    Data_node* data_node = (Data_node*)data_list->get_node_cc();
    
    if (data_node->get_value_cc(attr_value, attr_label_a) != ISIP_TRUE) {
      error_handler_cc((char_1*)"calculate_bayes_left_prob_cc",
		       (char_1*)"unable get attribute value");
    }

    // continuous value so we can do straight math comparison
    // num < split_value_a go to left
    // num >= split_value_a go to right
    //
    if (attr_value < split_value_a) {
      if (data_node->get_item_cc(class_value, class_label_a) != ISIP_TRUE) {
	error_handler_cc((char_1*)"calculate_bayes_left_prob_cc",
		       (char_1*)"unable get class value");
      }
      
      left_count++;
    
      for (int_4 j=0; j<num_classes_a; j++) {
	if (strcmp((char*)class_value, (char*)class_names_d[j]) == 0) {
	  class_count[j]++;
	  break;
	}
      }
    }
    data_list = data_list->get_next_cc();
  }
  
  // calculate the left probability
  //
  float_4 left_prob = 0;
  if (left_count > 0) {
    for (int_4 i=0; i<num_classes_a; i++) {
      if (class_count[i] > 0) {
	left_prob += class_count[i] * log10((float_4)class_count[i]/left_count);
      }
    }
  }
  
  // clean up
  //
  if (class_value != (char_1*)NULL) {
    delete [] class_value;
    class_value = (char_1*)NULL;
  }

  if (class_count != (int_4*)NULL) {
    delete [] class_count;
    class_count = (int_4*)NULL;
  }
  
  // return maximum value 
  //
  return left_prob;
}


















