// file: $DT/class/tree/v1.0/dt_split_1.cc
//

// isip include files
//
#include "tree_node.h"
#include "attribute_node.h"
#include "tree.h"
#include "decision_tree.h"

#include <integral_constants.h>
#include "attribute_node_constants.h"
#include "tree_node_constants.h"
#include "decision_tree_constants.h"

// method: best_split_cc
//
// arguments:
//   int_4 dt_split_a: (input) constant defining splitting rule used
//   Tree_node* node_a: (input) node for which the split is being found
//   Tree_node* left_child_a: (input) left child of node_a
//   Tree_node* right_child_a: (input) right child of node_a
//
// return: logical_1 indicating success
//
// this method determines the best split (discrete or continuous) for a node
// and then applies this split
//
//
logical_1 Decision_tree::best_split_cc(int_4 dt_split_a, Tree_node* node_a,
				       Tree_node* left_child_a,
				       Tree_node* right_child_a) {

  // place to store the maximum splits found for either discrete or cont
  //
  Attribute_node* max_attr;
  float_4 cont_split;
  logical_1 best_split;
  
  // find the best split
  //
  int_4* mask = new int_4[DT_MAX_NUM_OF_ATTR_VALUES];
  if (calculate_split_vector_cc(&max_attr, cont_split, dt_split_a,
				mask, node_a) == ISIP_TRUE) {

    // check for purity and apply the best split found
    //
    best_split = apply_best_split_cc(max_attr, mask, cont_split,
				     node_a, left_child_a, right_child_a);
    
  }
  else {

    // no need to split if no max attribute found
    //
    best_split = ISIP_FALSE;
  }

  // clean up
  //
  if (mask != (int_4*)NULL) {
    delete [] mask;
    mask = (int_4*)NULL;
  }
  
  // exit gracefully
  //
  return best_split;
}
