// file: $DT/class/tree/v1.0/dt_split_9.cc
//

// isip include files
//
#include "tree_node.h"
#include "attribute_node.h"
#include "tree.h"    
#include "decision_tree.h"

#include <integral_constants.h>
#include "tree_node_constants.h"
#include "attribute_node_constants.h"
#include "decision_tree_constants.h"

// method: calculate_discrete_split_cc
//
// arguments:
//  Attribute_node** attr_node_a: (input/output) attribute being split on
//  float_4& cont_split_a: (input/output) cont value being split on
//  int_4 dt_split_a: (input) type of splitting rule used
//  int_4* mask_a: (input/output) mask of best split
//  Tree_node* node_a: (input) node for which the split is being found
//
// return: logical_1 indicating if there is a best split attribute
//
// this method determines the best discrete for a node and then
// applies this split
//
//
logical_1 Decision_tree::calculate_split_vector_cc(Attribute_node** attr_node_a,
						   float_4& cont_split_a,
						   int_4 dt_split_a,
						   int_4* mask_a,
						   Tree_node* node_a) {

  int_4** mask_vector = new int_4*[num_of_attributes_d];
  float_4* cont_vector = new float_4[num_of_attributes_d];
  float_4* calc_vector = new float_4[num_of_attributes_d];
  int_4 max_index;
  logical_1 found_best = ISIP_FALSE;

  // declare memory location to store the gain for each attribute,
  // it's only used for IND's gain ratio splitting
  //
  float_4* gain_vector = new float_4[num_of_attributes_d];
  
  // determine the splitting rule and generate the vector of values
  //
  if (dt_split_a == DT_SPLIT_ENT) {
    calculate_vector_entropy_cc (mask_vector, cont_vector,
				 calc_vector, node_a);
    find_max_split_cc (calc_vector, max_index);
  }
  else if (dt_split_a == DT_SPLIT_BAYES) {
    calculate_vector_bayes_cc (mask_vector, cont_vector,
			       calc_vector, node_a);
    find_max_split_cc (calc_vector, max_index);
  }
  else if (dt_split_a == DT_SPLIT_GAINRATIO) {
    calculate_vector_gainratio_cc (mask_vector, cont_vector,
				   calc_vector, node_a);
    find_max_split_cc (calc_vector, max_index);
  }
  else if (dt_split_a == DT_SPLIT_IND_GAINRATIO) {
    calculate_vector_ind_gainratio_cc(mask_vector, cont_vector,
				      calc_vector, gain_vector,
				      node_a);
    find_max_split_cc (calc_vector, gain_vector, max_index);
  }

  if (max_index != DT_MAX_ATTR_NOT_FOUND) {

    // set return value
    //
    found_best = ISIP_TRUE;
    
    // retrieve the best attribute to split on
    //
    get_attribute_node_cc(max_index, attr_node_a);
    
    // store information related to splitting at this attribute
    //
    int_4 type_a;
    int_4 num_attr_values_a;
    Attribute_node* tmp_attr_a = *attr_node_a;
    
    tmp_attr_a->get_type_cc(type_a);
    
    if (type_a == ATTR_TYPE_DISCRETE) {
      tmp_attr_a->get_num_values_cc(num_attr_values_a);
      for (int_4 i=0; i<num_attr_values_a; i++) {
	mask_a[i] = mask_vector[max_index][i];
      }
    }
    else {
      cont_split_a = cont_vector[max_index];
    }
  }
  
  // clean up
  //
  if (mask_vector != (int_4**)NULL) {
    for (int_4 i=0; i<num_of_attributes_d; i++) {
      if (mask_vector[i] != (int_4*)NULL) {
	delete [] mask_vector[i];
	mask_vector[i] = (int_4*)NULL;
      }
    }
    delete [] mask_vector;
    mask_vector = (int_4**)NULL;
  }

  if (cont_vector != (float_4*)NULL) {
    delete [] cont_vector;
    cont_vector = (float_4*)NULL;
  }

  if (calc_vector != (float_4*)NULL) {
    delete [] calc_vector;
    calc_vector = (float_4*)NULL;
  }

  if (gain_vector != (float_4*)NULL) {
    delete [] gain_vector;
    gain_vector = (float_4*)NULL;
  }
  
  // exit gracefully
  //
  return found_best;
}

