// file: $DT/class/decision_tree/v1.0/decision_tree.h
//
 
// make sure the definitions are made only once
//
#ifndef __ISIP_DECISION_TREE
#define __ISIP_DECISION_TREE

// isip include files
//
#ifndef __ISIP_INTEGRAL
#include <integral.h>
#endif

// forward class declaration
//
class Data_node;
class Attribute_node;
class Tree_node;
class Linked_list;
class Param_file;
class Tree;

// Tree: a class that describes a decision tree.
//
class Decision_tree {
  
  //---------------------------------------------------------------------------
  //
  // protected data
  //
  //---------------------------------------------------------------------------
protected:
  
  // declare a variable for debugging
  // specify debug level (ISIP_DEBUG_NONE, ISIP_DEBUG_BRIEF, etc.)
  //
  int_4 debug_level_d;

  // mode
  //
  
  // specify train or test (DT_MODE_TRAIN, DT_MODE_TEST)
  //
  int_4 mode_d;

  // specify splitting rule (DT_SPLIT_ENT, DT_SPLIT_BAYES, DT_SPLIT_GAINRATIO)
  //
  int_4 mode_split_d;

  // specify attribute splitting search method (DT_SEARCH_COMPLETE,
  // DT_SEARCH_FAST)
  //
  int_4 mode_search_d;

  // specify pruning rule (DT_PRUNE_PESSIMISTIC, DT_PRUNE_COSTCOMPLEXITY)
  //
  int_4 mode_prune_d;

  // specify smoothing rule (DT_SMOOTH_BAYES)
  //
  int_4 mode_smooth_d;

  // specify display mode
  //
  int_4 mode_display_d;
  
  char_1* param_fname_d;               // specify name of paramter file
  char_1* attr_fname_d;                // specify name of attribute file
  char_1* data_fname_d;                // specify name of data file
  char_1* tree_fname_d;                // specify name of tree file
  
  // number of attributes read
  //
  int_4 num_of_attributes_d;

  // confidence limit
  //
  float_4 confidence_limit_d;

  // prior model for cut-point of real-valued attributes
  //
  int_4 prior_model_d;

  // dirichlet prior (alpha in the equation) for splitting
  //
  float_4 dirichlet_prior_d;

  // the class node
  //
  Attribute_node* class_node_d;

  // the attribute information
  //
  Linked_list* attribute_list_d;

  // the data information
  //
  Linked_list* data_list_d;

  // prior probability of each classes
  //
  int_4 num_classes_d;
  char_1** class_names_d;
  float_4* class_prior_probs_d;
  
  // FOR TESTING, REMOVE LATER
  //
  int_4 num_nodes_d;

  // a class that facilitates tree functions (traverse, parent, child,
  // insert, delete, etc.)
  //
  Tree* tree_d;  
  
  //---------------------------------------------------------------------------
  //
  // public members
  //
  //---------------------------------------------------------------------------
public:

  // required methods for debugging
  //
  char_1* name_cc();
  volatile void error_handler_cc(char_1* method_name, char_1* message);
  logical_1 debug_cc(FILE* fp, char_1* message);
  logical_1 set_debug_cc(int_4 debug_level);
  int_4 size_cc();

  // destructor/constructor
  //
  ~Decision_tree();
  Decision_tree();

  // get value methods
  //
  int_4 get_mode_cc ();
  int_4 get_display_mode_cc ();
  logical_1 get_attribute_node_cc(int_4 index, Attribute_node** attr);

  // i/o methods
  //
  logical_1 read_tree_cc(char_1* fname, Tree* tree);

  // load parameter, attribute file, and data file
  //
  logical_1 load_params_cc(char_1* fname);
  logical_1 load_attributes_cc(char_1* fname);
  logical_1 load_data_cc(char_1* fname);

  // display all attribute nodes, data file nodes functions
  //
  logical_1 display_attributes_cc();
  logical_1 display_data_cc();

  // display the tree for viewing according to display_level
  // (DT_DISPLAY_BASIC, DT_DISPLAY_COUNT, DT_DISPLAY_PROB)
  //
  logical_1 display_cc (FILE* fp, int_4 display_level);   

  // save the tree that is built to a file
  //
  logical_1 write_cc (char_1* fname);               
  
  // computation methods

  // call train_cc or test_cc depending on mode_d
  //
  logical_1 compute_cc(char_1* data_fname, char_1* result_fname); 

  //---------------------------------------------------------------------------
  //
  // private members
  //
  //---------------------------------------------------------------------------
private:

  // splitting methods
  //
  logical_1 split_cc(int_4 dt_split_a, Tree_node* node_a=(Tree_node*)NULL);
  logical_1 best_split_cc(int_4 dt_split_a, Tree_node* node_a,
			  Tree_node* left_child, Tree_node* right_child);
  logical_1 calculate_split_vector_cc(Attribute_node** attr_node_a,
				      float_4& cont_split,
				      int_4 dt_split_a, int_4* mask_a,
				      Tree_node* node_a);
  logical_1 find_max_split_cc (float_4* calc, float_4* gain, int_4& max_index);
  logical_1 find_max_split_cc (float_4* calc, int_4& max_index);

  logical_1 apply_best_split_cc(Attribute_node* attr, int_4* mask,
				float_4 cont_split, Tree_node* node,
				Tree_node* left_child, Tree_node* right_child);
  logical_1 apply_split_cc(float_4* left_prob, float_4* right_prob,
			   int_4* mask_a, Attribute_node* attr_node_a,
			   Tree_node* parent_a, Tree_node* left_a,
			   Tree_node* right_a);
  logical_1 apply_split_cc(float_4* left_prob, float_4* right_prob,
			   float_4 split_a, Attribute_node* attr_node_a,
			   Tree_node* parent_a, Tree_node* left_a,
			   Tree_node* right_a);
  int_4 classify_data_node_cc(int_4* mask_a, Attribute_node* attr_node_a,
			      Data_node* data_a);
  int_4 classify_data_node_cc(float_4 split_a, Attribute_node* attr_node_a,
			      Data_node* data_a);
  logical_1 not_pure_cc( Tree_node* node_a );
  logical_1 purity_check_cc (int_4* mask_a, Tree_node* node,
			     Attribute_node* attr_node);
  logical_1 purity_check_cc (float_4 split, Tree_node* node,
			     Attribute_node* attr_node);

  // methods to store the split values in the children
  //
  logical_1 set_children_cc(Tree_node* left_child, Tree_node* right_child,
			    Tree_node* node, float_4* left_prob,
			    float_4* right_prob);
			     
  // methods for calculating and storing splitting values and related
  // information
  //
  logical_1 calculate_vector_entropy_cc(int_4** mask_vector,
					float_4* cont_vector,
					float_4* calc_vector, Tree_node* node);
  logical_1 calculate_vector_bayes_cc(int_4** mask_vector,
				      float_4* cont_vector,
				      float_4* calc_vector, Tree_node* node);
  logical_1 calculate_vector_gainratio_cc(int_4** mask_vector,
					  float_4* cont_vector,
					  float_4* calc_vector,
					  Tree_node* node);
  logical_1 calculate_vector_ind_gainratio_cc(int_4** mask_vector,
					      float_4* cont_vector,
					      float_4* calc_vector,
					      float_4* gain_vector,
					      Tree_node* node);

  // split using maximum entropy
  //
  float_4 discrete_entropy_split_cc(int_4* max_mask_a, int_4 num_categories_a,
				    Attribute_node* attr_node_a,
				    Tree_node* node_a);
  float_4 continuous_entropy_split_cc( float_4& max_split_a,
				       Attribute_node* attr_node_a,
				       Tree_node* node_a); 
  float_4 calculate_entropy_prob_cc(int_4* mask_a, int_4 num_categories_a,
				    Attribute_node* attr_node_a,
				    Tree_node* node_a,int_4*& class_given_left,
				    int_4*& class_given_right,
				    int_4& left_count_a,int_4& right_count_a);
  float_4 calculate_entropy_prob_cc(float_4 split_a,
				    Attribute_node* attr_node_a,
				    Tree_node* node_a,int_4*& class_given_left,
				    int_4*& class_given_right,
				    int_4& left_count_a, int_4& right_count_a);

  // split using bayes rule
  //
  float_4 discrete_bayes_split_cc(int_4* max_mask, int_4 num_categories,
				  Attribute_node* attr_node, Tree_node* node);

  float_4 continuous_bayes_split_cc(float_4& max_split,
				    Attribute_node* attr_node,Tree_node* node);

  // using bayes and split on discrete attribute
  //
  float_4 calculate_bayes_left_prob_cc (int_4* mask,
					char_1* class_label,
					int_4 num_classes,
					char_1* attr_label,
					int_4 num_attrs,
					char_1** attr_values,
					Tree_node* node);

  float_4 calculate_bayes_right_prob_cc (int_4* mask,
					 char_1* class_label,
					 int_4 num_classes,
					 char_1* attr_label,
					 int_4 num_attrs,
					 char_1** attr_values,
					 Tree_node* node);

  // using bayes and split on continuous attribute
  //
  float_4 calculate_bayes_left_prob_cc (float_4 split_value,
					char_1* class_label,
					int_4 num_classes,
					char_1* attr_label,
					Tree_node* node);
  
  float_4 calculate_bayes_right_prob_cc (float_4 split_value,
					 char_1* class_label,
					 int_4 num_classes,
					 char_1* attr_label,
					 Tree_node* node);
  
  logical_1 calculate_prior_prob_cc();
  
  // split using gain ratio
  //
  float_4 discrete_gainratio_split_cc(int_4* max_mask_a,
				      int_4 num_categories_a,
				      Attribute_node* attr_node_a,
				      Tree_node* node_a);
  float_4 continuous_gainratio_split_cc(float_4& max_split_a,
					Attribute_node* attr_node_a,
					Tree_node* node_a);
  
  float_4 discrete_ind_gainratio_split_cc(int_4* max_mask_a,
					  int_4 num_categories_a,
					  Attribute_node* attr_node_a,
					  Tree_node* node_a,
					  float_4& gain_average_a);
  float_4 continuous_ind_gainratio_split_cc(float_4& max_split_a,
					    Attribute_node* attr_node_a,
					    Tree_node* node_a,
					    float_4& gain_average_a);
  
  // methods to calculate the information
  //
  float_4 calculate_node_info_cc(int_4 num_classes, Tree_node* node);
  float_4 calculate_info_cc(int_4* mask, int_4 num_classes,
			    int_4 num_categories,
			    Attribute_node* attr_node, Tree_node* node,
			    int_4& num_data_nodes, int_4& left, int_4& right);
  float_4 calculate_info_cc(float_4 split, int_4 num_classes,
			    Attribute_node* attr_node, Tree_node* node,
			    int_4& num_data_nodes, int_4& left, int_4& right);
  float_4 calculate_split_info_cc(int_4 left, int_4 right, int_4 num_data); 
  
  // method to calculate the entropy value for an event with the number of
  // counts and the total number of events
  //
  float_4 entropy_cc (int_4 count, int_4 total);
  
  // pruning methods
  //
  logical_1 prune_cc (int_4 dt_prune_a);
  logical_1 drop_thru_prune_cc (Tree_node* node);
  logical_1 apply_prune_cc (Tree_node* node);
  logical_1 prune_cc (char_1* output_fname);
  
  // pessimistic pruning methods
  //
  logical_1 prune_pessimistic_cc();
  logical_1 calculate_pessimistic_prune_cc(Tree_node* node);
  logical_1 calculate_pessimistic_prune_cc(Tree_node* node,
					   Tree_node* left_child,
					   Tree_node* right_child,
					   int_4& max_index);
  
  // smoothing methods
  //
  //logical_1 smooth_bayes_cc();         // smooth using bayes rule

  // helper functions for splitting and pruning
  //
  logical_1 calculate_loop_values_cc(int_4& init, int_4& iter,
				     int_4 num_categories);
  logical_1 permute_cc( int_4 num_a, int_4* mask_a, int_4 num_cat_a );
  logical_1 mask_single_cc(int_4 num_a, int_4* mask_a, int_4 num_cat_a);

  // method to calculate the upper binomial confidence limit for use
  // in C4 pruning
  //
  float_4 confidence_limit_cc(int_4 num_of_population, int_4 num_of_errors);

  // train and test methods
  //
  logical_1 train_cc(char_1* output_fname);
  logical_1 test_cc(char_1* output_fname);

  // method to parse the parameter file
  //
  logical_1 set_param_cc(char_1* param, char_1* value);
  
  // methods to parse the attribute and data files and check the
  // information
  //
  logical_1 pre_load_attributes_cc(char_1* fname, int_4& num_lines_read,
				   char_1** label_list, char_1** value_list);
  logical_1 parse_data_cc(char_1* buffer, int_4& num_attr,
			  char_1** attr_list);
  logical_1 check_data_cc(int_4 num_item, char_1** data);
  logical_1 parse_data_item_cc(char_1** data_item, int_4 num_attr,
			       char_1* desc, char_1** label,
			       char_1** value);
  logical_1 pre_parse_file_cc(char_1 *buffer, int_4& length,
			      int_4& num_values);

  // methods called by parsing code to tokenize items
  //
  logical_1 pre_tokenize_cc (char_1** lines, int_4& num_tokens,
			     char_1* buffer, char_1 token);
  logical_1 tokenize_line_cc(char_1* buffer, int_4 num_lines_read,
			     char_1** attr_list);
  logical_1 tokenize_colon_cc(char_1* buffer, char_1* label,
			      char_1* attr_valuelist);

  // method to generate class node and attribute nodes from attribute
  // file
  //
  logical_1 generate_nodes_cc(int_4 num_items, char_1** label_list,
			      char_1** value_list);

  // method to generate class node and tree classes
  //
  logical_1 generate_class_node_cc(char_1* label, char_1* values);
  logical_1 generate_tree_classes_cc(int_4 num_classes, char_1** classes);

  // method to generate attribute node
  //
  logical_1 generate_attr_node_cc(char_1* label, char_1* values);

  // method to generate data node
  //
  logical_1 generate_data_node_cc(char_1** data_item, int_4 num_attr);

  // methods to find specific information for an attribute from the
  // attribute file
  //
  logical_1 find_type_cc(char_1* attr_valuelist, int_4& type_a);
  logical_1 find_range_cc(char_1* attr_valuelist, int_4 type, int_4& range,
			  float_4& min, float_4& max);
  logical_1 find_minmax_cc(char_1 *buffer_a, float_4& min_a, float_4& max_a);

  // method to manually create discrete data from min, max values
  //
  logical_1 manual_range_cc(char_1** attr_values, int_4 min, int_4 max);

  // methods for string conversions
  //
  logical_1 convert_string_cc(int_4& value_a, char_1* string_a);
  logical_1 convert_string_cc(float_4& value_a, char_1* string_a);
  logical_1 convert_to_value_cc(int_4 num, char_1** label,
				char_1** item, float_4* value);

  // method to search for an attribute by label
  //
  logical_1 search_attribute_cc(char_1* label, Attribute_node* node);
  
  // method to parse the tree file read in
  //
  logical_1 parse_tree_cc( int_4 &level_a, Tree_node* node_a, char_1* input_a,
			   char_1* count_a, char_1* prob_a);
  
  // load the data for tree building or load the tree for evaluation
  //
  logical_1 load_tree_cc();           // load_params_cc calls this method

  // utility methods 
  //

  // quicksort
  //
  logical_1 qsort_cc(int_4* index, float_4* values, int_4 num_items);
  static int compare_float_cc(float_4* num1_a, float_4* num2_a);

  // files used for classifying in test mode
  //
  logical_1 classify_data_cc (char_1* fname);
  logical_1 drop_thru_tree_cc ( Data_node* data_a, Tree_node* tree_node_a,
				char_1* class_a ) ;

};

// end of file
//
#endif



